from spaceone.core.service import *
from spaceone.core import utils
from spaceone.inventory.error import *
from spaceone.inventory.model.cloud_service_type_model import CloudServiceType
from spaceone.inventory.manager.cloud_service_type_manager import CloudServiceTypeManager


_KEYWORD_FILTER = ['cloud_service_type_id', 'name', 'group', 'service_code']


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class CloudServiceTypeService(BaseService):

    def __init__(self, metadata):
        super().__init__(metadata)
        self.cloud_svc_type_mgr: CloudServiceTypeManager = self.locator.get_manager('CloudServiceTypeManager')

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    def create(self, params):
        """
        Args:
            params (dict): {
                    'name': 'str',
                    'group': 'str',
                    'provider': 'str',
                    'service_code': 'str',
                    'is_primary': 'bool',
                    'is_major': 'bool',
                    'resource_type': 'str',
                    'metadata': 'dict',
                    'labels': 'list,
                    'tags': 'list or dict',
                    'domain_id': 'str'
                }

        Returns:
            cloud_service_type_vo (object)
        """

        return self.create_resource(params)

    @check_required(['name', 'provider', 'group', 'domain_id'])
    def create_resource(self, params):
        params['updated_by'] = self.transaction.get_meta('collector_id') or 'manual'

        provider = params.get('provider', self.transaction.get_meta('secret.provider'))

        if provider:
            params['provider'] = provider

        if 'tags' in params:
            if isinstance(params['tags'], list):
                params['tags'] = utils.tags_to_dict(params['tags'])

        params['resource_type'] = params.get('resource_type', 'inventory.CloudService')

        params['ref_cloud_service_type'] = f'{params["domain_id"]}.{params["provider"]}.' \
                                           f'{params["group"]}.{params["name"]}'

        params['cloud_service_type_key'] = f'{params["provider"]}.{params["group"]}.{params["name"]}'

        return self.cloud_svc_type_mgr.create_cloud_service_type(params)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    def update(self, params):
        """
        Args:
            params (dict): {
                    'cloud_service_type_id': 'str',
                    'service_code': 'str',
                    'is_primary': 'bool',
                    'is_major': 'bool',
                    'resource_type': 'str',
                    'metadata': 'dict',
                    'labels': 'list',
                    'tags': 'list or dict',
                    'domain_id': 'str'
                }

        Returns:
            cloud_service_type_vo (object)
        """

        return self.update_resource(params)

    @check_required(['cloud_service_type_id', 'domain_id'])
    def update_resource(self, params):
        if 'tags' in params:
            if isinstance(params['tags'], list):
                params['tags'] = utils.tags_to_dict(params['tags'])

        params['updated_by'] = self.transaction.get_meta('collector_id') or 'manual'

        provider = params.get('provider', self.transaction.get_meta('secret.provider'))
        domain_id = params['domain_id']

        cloud_svc_type_vo = self.cloud_svc_type_mgr.get_cloud_service_type(params['cloud_service_type_id'],
                                                                           domain_id)

        if not cloud_svc_type_vo.cloud_service_type_key:
            params['cloud_service_type_key'] = f'{cloud_svc_type_vo.provider}.{cloud_svc_type_vo.group}.' \
                                               f'{cloud_svc_type_vo.name}'

        return self.cloud_svc_type_mgr.update_cloud_service_type_by_vo(params, cloud_svc_type_vo)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    def delete(self, params):
        """
        Args:
        params (dict): {
            'cloud_service_type_id': 'str',
            'domain_id': 'str'
        }
        Returns:
            None
        """
        self.delete_resource(params)

    @check_required(['cloud_service_type_id', 'domain_id'])
    def delete_resource(self, params):
        cloud_svc_type_vo = self.cloud_svc_type_mgr.get_cloud_service_type(params['cloud_service_type_id'],
                                                                           params['domain_id'])

        self.cloud_svc_type_mgr.delete_cloud_service_type_by_vo(cloud_svc_type_vo)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['cloud_service_type_id', 'domain_id'])
    def get(self, params):
        """
        Args:
            params (dict): {
                    'cloud_service_type_id': 'str',
                    'domain_id': 'str',
                    'only': 'list'
                }

        Returns:
            cloud_service_type_vo (object)

        """

        return self.cloud_svc_type_mgr.get_cloud_service_type(params['cloud_service_type_id'], params['domain_id'],
                                                              params.get('only'))

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['domain_id'])
    @append_query_filter(['cloud_service_type_id', 'name', 'provider', 'group', 'cloud_service_type_key',
                          'service_code', 'is_primary', 'is_major', 'resource_type', 'domain_id'])
    @append_keyword_filter(_KEYWORD_FILTER)
    def list(self, params):
        """
        Args:
            params (dict): {
                    'cloud_service_type_id': 'str',
                    'name': 'str',
                    'group': 'str',
                    'provider': 'str',
                    'cloud_service_type_key': 'str',
                    'service_code': 'str',
                    'is_primary': 'str',
                    'is_major': 'str',
                    'resource_type': 'str',
                    'domain_id': 'str',
                    'query': 'dict (spaceone.api.core.v1.Query)'
                }

        Returns:
            results (list)
            total_count (int)

        """

        return self.cloud_svc_type_mgr.list_cloud_service_types(params.get('query', {}))

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['query', 'domain_id'])
    @append_query_filter(['domain_id'])
    @append_keyword_filter(_KEYWORD_FILTER)
    def stat(self, params):
        """
        Args:
            params (dict): {
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)'
            }

        Returns:
            values (list) : 'list of statistics data'

        """

        query = params.get('query', {})
        return self.cloud_svc_type_mgr.stat_cloud_service_types(query)



from spaceone.api.inventory.v1 import collector_pb2, collector_pb2_grpc
from spaceone.core.pygrpc import BaseAPI


class Collector(BaseAPI, collector_pb2_grpc.CollectorServicer):

    pb2 = collector_pb2
    pb2_grpc = collector_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('CollectorService', metadata) as collector_service:
            return self.locator.get_info('CollectorInfo', collector_service.create(params))

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('CollectorService', metadata) as collector_service:
            return self.locator.get_info('CollectorInfo', collector_service.update(params))

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('CollectorService', metadata) as collector_service:
            collector_service.delete(params)
            return self.locator.get_info('EmptyInfo')

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('CollectorService', metadata) as collector_service:
            return self.locator.get_info('CollectorInfo', collector_service.get(params))

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('CollectorService', metadata) as collector_service:
            collector_vos, total_count = collector_service.list(params)
            return self.locator.get_info('CollectorsInfo', collector_vos, total_count, minimal=self.get_minimal(params))

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('CollectorService', metadata) as collector_service:
            return self.locator.get_info('StatisticsInfo', collector_service.stat(params))

    def collect(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('CollectorService', metadata) as collector_service:
            return self.locator.get_info('JobInfo', collector_service.collect(params))

    def update_plugin(self, request, context):
        params, metadata = self.parse_request(request, context)
        with self.locator.get_service('CollectorService', metadata) as collector_service:
            return self.locator.get_info('CollectorInfo', collector_service.update_plugin(params))

    def verify_plugin(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('CollectorService', metadata) as collector_service:
            collector_service.verify_plugin(params)
            return self.locator.get_info('EmptyInfo')



import datetime
import logging
from spaceone.core.locator import Locator
from spaceone.core.scheduler import HourlyScheduler
from spaceone.inventory.lib.scheduler import init_count, update_token, get_domain_id_from_token
from spaceone.inventory.service.collector_service import CollectorService


__all__ = ['InventoryHourlyScheduler']

_LOGGER = logging.getLogger(__name__)


class InventoryHourlyScheduler(HourlyScheduler):
    def __init__(self, queue, interval, minute=':00'):
        super().__init__(queue, interval, minute)
        self.count = init_count()
        self.locator = Locator()
        self.TOKEN = update_token()
        self.domain_id = get_domain_id_from_token(self.TOKEN)

    def create_task(self):
        return [self._create_job_request(collector_vo) for collector_vo in self.list_schedule_collectors()]

    def list_schedule_collectors(self):
        try:
            self.check_count()
            collector_svc: CollectorService = self.locator.get_service(CollectorService)
            collector_vos, total = collector_svc.scheduled_collectors({'schedule': {'hour': self.count['hour']}})
            _LOGGER.debug(f'[push_token] scheduled collectors count: {total}')
            return collector_vos
        except Exception as e:
            _LOGGER.error(e)
            return []

    def check_count(self):
        current_time = datetime.datetime.utcnow()
        hour = current_time.hour

        if (self.count['hour'] + self.config) % 24 != hour:
            if self.count['hour'] == hour:
                _LOGGER.error('[check_count] duplicated call in the same time')
            else:
                _LOGGER.error('[check_count] missing time')

        self.count.update({
            'previous': current_time,
            'index': self.count['index'] + 1,
            'hour': hour,
            'started_at': current_time
        })

    def _update_count_ended_at(self):
        cur = datetime.datetime.utcnow()
        self.count['ended_at'] = cur

    def _create_job_request(self, collector_vo):
        schedule_job = {
            'locator': 'SERVICE',
            'name': 'CollectorService',
            'metadata': self._get_metadata(),
            'method': 'collect',
            'params': {
                'params': {
                    'collector_id': collector_vo.collector_id,
                    'collect_mode': 'ALL',
                    'filter': {},
                    'domain_id': collector_vo.domain_id
                }
            }
        }

        return {
            'name': 'inventory_collect_schedule',
            'version': 'v1',
            'executionEngine': 'BaseWorker',
            'stages': [schedule_job]
        }

    def _get_metadata(self):
        return {
            'token': self.TOKEN,
            'service': 'inventory',
            'resource': 'Collector',
            'verb': 'collect',
            'domain_id': self.domain_id
        }



from spaceone.inventory.model.region_model import Region
from spaceone.inventory.model.collector_model import Collector
from spaceone.inventory.model.job_model import Job
from spaceone.inventory.model.job_task_model import JobTask
from spaceone.inventory.model.cloud_service_model import CloudService
from spaceone.inventory.model.cloud_service_type_model import CloudServiceType
from spaceone.inventory.model.cloud_service_query_set_model import CloudServiceQuerySet
from spaceone.inventory.model.cloud_service_stats_model import CloudServiceStats, MonthlyCloudServiceStats, \
    CloudServiceStatsQueryHistory
from spaceone.inventory.model.collection_state_model import CollectionState
from spaceone.inventory.model.reference_resource_model import ReferenceResource
from spaceone.inventory.model.resource_group_model import ResourceGroup
from spaceone.inventory.model.record_model import Record
from spaceone.inventory.model.note_model import Note
from spaceone.inventory.model.collector_rule_model import CollectorRule



from mongoengine import *
from spaceone.core.model.mongo_model import MongoModel


class Resource(EmbeddedDocument):
    resource_type = StringField()
    filter = ListField(DictField())
    keyword = StringField(default=None, null=True)


class ResourceGroup(MongoModel):
    resource_group_id = StringField(max_length=40, generate_id='rsc-grp', unique=True)
    name = StringField(max_length=255)
    resources = ListField(EmbeddedDocumentField(Resource))
    options = DictField()
    tags = DictField()
    project_id = StringField(max_length=255)
    domain_id = StringField(max_length=255)
    created_at = DateTimeField(auto_now_add=True)

    meta = {
        'updatable_fields': [
            'name',
            'resources',
            'project_id',
            'options',
            'tags'
        ],
        'minimal_fields': [
            'resource_group_id',
            'name',
            'project_id'
        ],
        'change_query_keys': {
            'user_projects': 'project_id'
        },
        'ordering': [
            'name'
        ],
        'indexes': [
            # 'resource_group_id',
            'resources.resource_type',
            'project_id',
            'domain_id',
        ]
    }



from spaceone.api.inventory.v1 import cloud_service_query_set_pb2, cloud_service_query_set_pb2_grpc
from spaceone.core.pygrpc import BaseAPI


class CloudServiceQuerySet(BaseAPI, cloud_service_query_set_pb2_grpc.CloudServiceQuerySetServicer):

    pb2 = cloud_service_query_set_pb2
    pb2_grpc = cloud_service_query_set_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('CloudServiceQuerySetService', metadata) as cloud_svc_query_set_service:
            return self.locator.get_info('CloudServiceQuerySetInfo', cloud_svc_query_set_service.create(params))

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('CloudServiceQuerySetService', metadata) as cloud_svc_query_set_service:
            return self.locator.get_info('CloudServiceQuerySetInfo', cloud_svc_query_set_service.update(params))

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('CloudServiceQuerySetService', metadata) as cloud_svc_query_set_service:
            cloud_svc_query_set_service.delete(params)
            return self.locator.get_info('EmptyInfo')

    def run(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('CloudServiceQuerySetService', metadata) as cloud_svc_query_set_service:
            cloud_svc_query_set_service.run(params)
            return self.locator.get_info('EmptyInfo')

    def test(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('CloudServiceQuerySetService', metadata) as cloud_svc_query_set_service:
            return self.locator.get_info('AnalyzeInfo', cloud_svc_query_set_service.test(params))

    def enable(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('CloudServiceQuerySetService', metadata) as cloud_svc_query_set_service:
            return self.locator.get_info('CloudServiceQuerySetInfo', cloud_svc_query_set_service.enable(params))

    def disable(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('CloudServiceQuerySetService', metadata) as cloud_svc_query_set_service:
            return self.locator.get_info('CloudServiceQuerySetInfo', cloud_svc_query_set_service.disable(params))

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('CloudServiceQuerySetService', metadata) as cloud_svc_query_set_service:
            return self.locator.get_info('CloudServiceQuerySetInfo', cloud_svc_query_set_service.get(params))

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('CloudServiceQuerySetService', metadata) as cloud_svc_query_set_service:
            cloud_svc_type_vos, total_count = cloud_svc_query_set_service.list(params)
            return self.locator.get_info('CloudServiceQuerySetsInfo',
                                         cloud_svc_type_vos,
                                         total_count,
                                         minimal=self.get_minimal(params))

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('CloudServiceQuerySetService', metadata) as cloud_svc_query_set_service:
            return self.locator.get_info('StatisticsInfo', cloud_svc_query_set_service.stat(params))



from spaceone.inventory.connector.identity_connector import IdentityConnector
from spaceone.inventory.connector.plugin_connector import PluginConnector
from spaceone.inventory.connector.secret_connector import SecretConnector
from spaceone.inventory.connector.collector_connector import CollectorPluginConnector
from spaceone.inventory.connector.repository_connector import RepositoryConnector
from spaceone.inventory.connector.config_connector import ConfigConnector



from spaceone.core.error import *


class ERROR_INVALID_PRIMARY_IP_ADDRESS(ERROR_INVALID_ARGUMENT):
    _message = 'Primary IP address does not exist in NICs.'


class ERROR_RESOURCE_ALREADY_DELETED(ERROR_INVALID_ARGUMENT):
    _message = '{resource_type} has already been deleted. ({resource_id})'



import functools
from spaceone.api.inventory.v1 import job_task_pb2, collector_pb2
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils
from spaceone.inventory.model.job_task_model import JobTask

__all__ = ['JobTaskInfo', 'JobTasksInfo']


def ErrorInfo(error):
    info = {
        'error_code': error.error_code,
        'message': error.message,
        'additional': change_struct_type(error.additional)
    }
    return collector_pb2.ErrorInfo(**info)


def JobTaskInfo(job_task_vo: JobTask, minimal=False):
    info = {
        'job_task_id': job_task_vo.job_task_id,
        'status': job_task_vo.status,
        'created_count': job_task_vo.created_count,
        'updated_count': job_task_vo.updated_count,
        'disconnected_count': job_task_vo.disconnected_count,
        'deleted_count': job_task_vo.deleted_count,
        'failure_count': job_task_vo.failure_count,
        'job_id': job_task_vo.job_id,
        'created_at': utils.datetime_to_iso8601(job_task_vo.created_at),
        'started_at': utils.datetime_to_iso8601(job_task_vo.started_at),
        'finished_at': utils.datetime_to_iso8601(job_task_vo.finished_at)
    }

    if not minimal:
        info.update({
            'errors': list(map(functools.partial(ErrorInfo), job_task_vo.errors)),
            'secret_id': job_task_vo.secret_id,
            'provider': job_task_vo.provider,
            'service_account_id': job_task_vo.service_account_id,
            'project_id': job_task_vo.project_id,
            'domain_id': job_task_vo.domain_id,
        })

    return job_task_pb2.JobTaskInfo(**info)


def JobTasksInfo(vos, total_count, **kwargs):
    return job_task_pb2.JobTasksInfo(results=list(map(functools.partial(JobTaskInfo, **kwargs), vos)), total_count=total_count)



import functools
from spaceone.api.inventory.v1 import change_history_pb2
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils
from spaceone.inventory.model.record_model import Record, RecordDiff

__all__ = ['RecordInfo', 'ChangeHistoryInfo']


def RecordDiffInfo(diff_vo: RecordDiff):
    # info = {
    #     'key': diff_vo.key,
    #     'before': change_value_type(diff_vo.before),
    #     'after': change_value_type(diff_vo.after),
    #     'type': diff_vo.type,
    # }
    #
    # return change_history_pb2.RecordDiff(**info)
    return {
        'key': diff_vo.key,
        'before': diff_vo.before,
        'after': diff_vo.after,
        'type': diff_vo.type
    }


def RecordInfo(record_vo: Record, minimal=False):
    info = {
        'record_id': record_vo.record_id,
        'cloud_service_id': record_vo.cloud_service_id,
        'action': record_vo.action,
        'diff_count': record_vo.diff_count,
        'user_id': record_vo.user_id,
        'collector_id': record_vo.collector_id,
        'job_id': record_vo.job_id,
        'created_at': utils.datetime_to_iso8601(record_vo.created_at),
    }

    if not minimal:
        info.update({
            # 'diff': list(map(RecordDiffInfo, record_vo.diff)),
            'diff': change_list_value_type(list(map(RecordDiffInfo, record_vo.diff))),
            'domain_id': record_vo.domain_id
        })

    return change_history_pb2.RecordInfo(**info)


def ChangeHistoryInfo(vos, total_count, **kwargs):
    return change_history_pb2.ChangeHistoryInfo(results=list(map(functools.partial(RecordInfo, **kwargs), vos)),
                                                total_count=total_count)



from spaceone.api.inventory.v1 import collector_rule_pb2, collector_rule_pb2_grpc
from spaceone.core.pygrpc import BaseAPI


class CollectorRule(BaseAPI, collector_rule_pb2_grpc.CollectorRuleServicer):

    pb2 = collector_rule_pb2
    pb2_grpc = collector_rule_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('CollectorRuleService', metadata) as collector_rule_service:
            return self.locator.get_info('CollectorRuleInfo', collector_rule_service.create(params))

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('CollectorRuleService', metadata) as collector_rule_service:
            return self.locator.get_info('CollectorRuleInfo', collector_rule_service.update(params))

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('CollectorRuleService', metadata) as collector_rule_service:
            collector_rule_service.delete(params)
            return self.locator.get_info('EmptyInfo')

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('CollectorRuleService', metadata) as collector_rule_service:
            return self.locator.get_info('CollectorRuleInfo', collector_rule_service.get(params))

    def change_order(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('CollectorRuleService', metadata) as collector_rule_service:
            return self.locator.get_info('CollectorRuleInfo', collector_rule_service.change_order(params))

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('CollectorRuleService', metadata) as collector_rule_service:
            collector_rule_vos, total_count = collector_rule_service.list(params)
            return self.locator.get_info('CollectorRulesInfo', collector_rule_vos, total_count, minimal=self.get_minimal(params))

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('CollectorRuleService', metadata) as collector_rule_service:
            return self.locator.get_info('StatisticsInfo', collector_rule_service.stat(params))



from mongoengine import *

from spaceone.core.model.mongo_model import MongoModel


class CollectorRuleCondition(EmbeddedDocument):
    key = StringField(required=True)
    value = StringField(required=True)
    operator = StringField(choices=('eq', 'contain', 'not', 'not_contain'))


class CollectorRuleOptions(EmbeddedDocument):
    stop_processing = BooleanField(default=False)


class CollectorRule(MongoModel):
    collector_rule_id = StringField(max_length=40, generate_id='collector-rule', unique=True)
    name = StringField(max_length=255, default='')
    order = IntField(required=True)
    conditions = ListField(EmbeddedDocumentField(CollectorRuleCondition), default=[])
    conditions_policy = StringField(max_length=20, choices=('ALL', 'ANY', 'ALWAYS'))
    actions = DictField(default={})
    rule_type = StringField(max_length=255, default='CUSTOM', choices=('MANAGED', 'CUSTOM'))
    options = EmbeddedDocumentField(CollectorRuleOptions, default=CollectorRuleOptions)
    tags = DictField(default={})
    collector = ReferenceField('Collector', reverse_delete_rule=CASCADE)
    collector_id = StringField(max_length=40)
    domain_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)

    meta = {
        'updatable_fields': [
            'name',
            'order',
            'conditions',
            'conditions_policy',
            'actions',
            'options',
            'tags'
        ],
        'minimal_fields': [
            'collector_rule_id',
            'name',
            'order',
            'rule_type',
            'collector_id'
        ],
        'ordering': ['order'],
        'indexes': [
            'order',
            'conditions_policy',
            'collector_id',
            'domain_id'
        ]
    }



import functools
from typing import List
from spaceone.api.inventory.v1 import collector_rule_pb2
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils

from spaceone.inventory.model.collector_rule_model import CollectorRule, CollectorRuleCondition, CollectorRuleOptions

__all__ = ['CollectorRuleInfo', 'CollectorRulesInfo']


def CollectorRuleConditionsInfo(condition_vos: List[CollectorRuleCondition]):
    if condition_vos is None:
        condition_vos = []

    conditions_info = []

    for vo in condition_vos:
        info = {
            'key': vo.key,
            'value': vo.value,
            'operator': vo.operator
        }

        conditions_info.append(collector_rule_pb2.CollectorRuleCondition(**info))

    return conditions_info


def CollectorRuleActionMatchRuleInfo(match_rule_data):
    if match_rule_data is None:
        return None

    info = {
        'source': match_rule_data.get('source'),
        'target': match_rule_data.get('target')
    }

    return collector_rule_pb2.MatchRule(**info)


def CollectorRuleActionsInfo(actions_data):
    if actions_data is None:
        return None
    else:
        info = {}

        for key, value in actions_data.items():
            if key in ['match_project', 'match_service_account']:
                info[key] = CollectorRuleActionMatchRuleInfo(value)
            elif key == 'add_additional_info':
                info[key] = change_struct_type(value)
            else:
                info[key] = value

        return collector_rule_pb2.CollectorRuleActions(**info)


def CollectorRuleOptionsInfo(vo: CollectorRuleOptions):
    if vo is None:
        return None
    else:
        info = {
            'stop_processing': vo.stop_processing
        }

        return collector_rule_pb2.CollectorRuleOptions(**info)


def CollectorRuleInfo(collector_rule_vo: CollectorRule, minimal=False):
    info = {
        'collector_rule_id': collector_rule_vo.collector_rule_id,
        'name': collector_rule_vo.name,
        'order': collector_rule_vo.order,
        'rule_type': collector_rule_vo.rule_type,
        'collector_id': collector_rule_vo.collector_id,
    }

    if not minimal:
        info.update({
            'conditions': CollectorRuleConditionsInfo(collector_rule_vo.conditions),
            'conditions_policy': collector_rule_vo.conditions_policy,
            'actions': CollectorRuleActionsInfo(collector_rule_vo.actions),
            'options': CollectorRuleOptionsInfo(collector_rule_vo.options),
            'tags': change_struct_type(collector_rule_vo.tags),
            'domain_id': collector_rule_vo.domain_id,
            'created_at': utils.datetime_to_iso8601(collector_rule_vo.created_at)
        })

    return collector_rule_pb2.CollectorRuleInfo(**info)


def CollectorRulesInfo(collector_rule_vos, total_count, **kwargs):
    return collector_rule_pb2.CollectorRulesInfo(results=list(
        map(functools.partial(CollectorRuleInfo, **kwargs), collector_rule_vos)), total_count=total_count)



from mongoengine import *

from spaceone.core.model.mongo_model import MongoModel


class PluginInfo(EmbeddedDocument):
    plugin_id = StringField(max_length=255)
    version = StringField(max_length=255)
    options = DictField()
    metadata = DictField()
    upgrade_mode = StringField(max_length=20, default='AUTO', choices=('AUTO', 'MANUAL'))

    def to_dict(self):
        return dict(self.to_mongo())


class SecretFilter(EmbeddedDocument):
    state = StringField(max_length=20, default='ENABLED', choices=('ENABLED', 'DISABLED'))
    secrets = ListField(StringField(max_length=40), defualt=None, null=True)
    service_accounts = ListField(StringField(max_length=40), default=None, null=True)
    schemas = ListField(StringField(max_length=40), default=None, null=True)
    exclude_secrets = ListField(StringField(max_length=40), defualt=None, null=True)
    exclude_service_accounts = ListField(StringField(max_length=40), default=None, null=True)
    exclude_schemas = ListField(StringField(max_length=40), default=None, null=True)

class Scheduled(EmbeddedDocument):
    state = StringField(max_length=20, default='ENABLED', choices=('ENABLED', 'DISABLED'))
    hours = ListField(default=None, null=True)


class Collector(MongoModel):
    collector_id = StringField(max_length=40, generate_id='collector', unique=True)
    name = StringField(max_length=255)
    provider = StringField(max_length=40, default=None, null=True)
    capability = DictField()
    plugin_info = EmbeddedDocumentField(PluginInfo, default=None, null=True)
    schedule = EmbeddedDocumentField(Scheduled, default=None, null=False)
    secret_filter = EmbeddedDocumentField(SecretFilter, default=None, null=True)
    priority = IntField(min_value=0, default=10, max_value=99)
    tags = DictField()
    domain_id = StringField(max_length=255)
    created_at = DateTimeField(auto_now_add=True)
    last_collected_at = DateTimeField()

    meta = {
        'updatable_fields': [
            'name',
            'plugin_info',
            'schedule',
            'secret_filter',
            'tags',
            'last_collected_at'
        ],
        'minimal_fields': [
            'collector_id',
            'name',
            'provider',
            'capability',
            'plugin_info'
        ],
        'change_query_keys': {
            'plugin_id': 'plugin_info.plugin_id'
        },
        'ordering': [
            'name'
        ],
        'indexes': [
            'provider',
            'priority',
            'domain_id',
        ]
    }



from mongoengine import *

from spaceone.core.model.mongo_model import MongoModel
from spaceone.inventory.model.cloud_service_type_model import CloudServiceType
from spaceone.inventory.model.region_model import Region


class CloudServiceStats(MongoModel):
    query_set_id = StringField(max_length=40, required=True)
    status = StringField(max_length=20, default='IN_PROGRESS', choices=('IN_PROGRESS', 'DONE'))
    values = DictField()
    unit = DictField()
    provider = StringField(max_length=255)
    cloud_service_group = StringField(max_length=255)
    cloud_service_type = StringField(max_length=255)
    ref_cloud_service_type = StringField(max_length=255)
    region_code = StringField(max_length=255, default=None, null=True)
    ref_region = StringField(max_length=255, default=None, null=True)
    account = StringField(max_length=255, default=None, null=True)
    additional_info = DictField()
    project_id = StringField(max_length=255, default=None, null=True)
    domain_id = StringField(max_length=40)
    created_year = StringField(max_length=20, required=True)
    created_month = StringField(max_length=20, required=True)
    created_date = StringField(max_length=20, required=True)

    meta = {
        'updatable_fields': [],
        'minimal_fields': [
            'query_set_id',
            'values',
            'unit',
            'provider',
            'cloud_service_group',
            'cloud_service_type',
            'project_id',
            'created_date'
        ],
        'change_query_keys': {
            'user_projects': 'project_id'
        },
        'reference_query_keys': {
            'ref_cloud_service_type': {
                'model': CloudServiceType,
                'foreign_key': 'ref_cloud_service_type'
            },
            'ref_region': {
                'model': Region,
                'foreign_key': 'ref_region'
            }
        },
        'indexes': [
            {
                "fields": ['domain_id', 'query_set_id', 'status', '-created_date', 'project_id'],
                "name": "COMPOUND_INDEX_FOR_SEARCH"
            },
            {
                "fields": ['domain_id', 'query_set_id', 'status', '-created_month'],
                "name": "COMPOUND_INDEX_FOR_SYNC_JOB"
            },
            {
                "fields": ['domain_id', 'query_set_id'],
                "name": "COMPOUND_INDEX_FOR_DELETE"
            }
        ]
    }


class MonthlyCloudServiceStats(MongoModel):
    query_set_id = StringField(max_length=40, required=True)
    status = StringField(max_length=20, default='IN_PROGRESS', choices=('IN_PROGRESS', 'DONE'))
    values = DictField()
    unit = DictField()
    provider = StringField(max_length=255)
    cloud_service_group = StringField(max_length=255)
    cloud_service_type = StringField(max_length=255)
    ref_cloud_service_type = StringField(max_length=255)
    region_code = StringField(max_length=255, default=None, null=True)
    ref_region = StringField(max_length=255, default=None, null=True)
    account = StringField(max_length=255, default=None, null=True)
    additional_info = DictField()
    project_id = StringField(max_length=255, default=None, null=True)
    domain_id = StringField(max_length=40)
    created_year = StringField(max_length=20)
    created_month = StringField(max_length=20)

    meta = {
        'updatable_fields': [],
        'change_query_keys': {
            'user_projects': 'project_id'
        },
        'reference_query_keys': {
            'ref_cloud_service_type': {
                'model': CloudServiceType,
                'foreign_key': 'ref_cloud_service_type'
            },
            'ref_region': {
                'model': Region,
                'foreign_key': 'ref_region'
            }
        },
        'indexes': [
            {
                "fields": ['domain_id', 'query_set_id', 'status', '-created_month', 'project_id'],
                "name": "COMPOUND_INDEX_FOR_SEARCH"
            },
            {
                "fields": ['domain_id', 'query_set_id', 'status', '-created_year'],
                "name": "COMPOUND_INDEX_FOR_SYNC_JOB"
            },
            {
                "fields": ['domain_id', 'query_set_id'],
                "name": "COMPOUND_INDEX_FOR_DELETE"
            }
        ]
    }


class CloudServiceStatsQueryHistory(MongoModel):
    query_hash = StringField(max_length=255)
    query_options = DictField(default={})
    query_set_id = StringField(max_length=40)
    domain_id = StringField(max_length=40)
    updated_at = DateTimeField(auto_now=True)

    meta = {
        'updatable_fields': [
            'updated_at'
        ],
        'indexes': [
            {
                "fields": ['domain_id', 'query_set_id', 'query_hash'],
                "name": "COMPOUND_INDEX_FOR_SEARCH"
            },
        ]
    }


from mongoengine import *
from datetime import datetime
from spaceone.core.model.mongo_model import MongoModel


class Error(EmbeddedDocument):
    error_code = StringField()
    message = StringField()
    additional = DictField()


class JobTask(MongoModel):
    job_task_id = StringField(max_length=40, generate_id='job_task', unique=True)
    status = StringField(max_length=20, default='PENDING',
                         choices=('PENDING', 'CANCELED', 'IN_PROGRESS', 'SUCCESS', 'FAILURE'))
    created_count = IntField(default=0)
    updated_count = IntField(default=0)
    deleted_count = IntField(default=0)
    disconnected_count = IntField(default=0)
    failure_count = IntField(default=0)
    total_count = IntField(default=0)
    errors = ListField(EmbeddedDocumentField(Error, default=None, null=True))
    job_id = StringField(max_length=40)
    secret_id = StringField(max_length=40)
    collector_id = StringField(max_length=40, default=None, null=True)
    provider = StringField(max_length=40, default=None, null=True)
    service_account_id = StringField(max_length=40, default=None, null=True)
    project_id = StringField(max_length=255, default=None, null=True)
    domain_id = StringField(max_length=255)
    created_at = DateTimeField(auto_now_add=True)
    started_at = DateTimeField(default=None, null=True)
    finished_at = DateTimeField(default=None, null=True)

    meta = {
        'updatable_fields': [
            'status',
            'secret_id',
            'provider',
            'service_account_id',
            'project_id',
            'created_count',
            'updated_count',
            'deleted_count',
            'disconnected_count',
            'failure_count',
            'errors',
            'started_at',
            'finished_at'
        ],
        'minimal_fields': [
            'job_task_id',
            'status',
            'created_count',
            'updated_count',
            'deleted_count',
            'disconnected_count',
            'failure_count',
            'job_id',
            'created_at',
            'started_at',
            'finished_at',
        ],
        'change_query_keys': {
            'user_projects': 'project_id'
        },
        'ordering': [
            '-created_at'
        ],
        'indexes': [
            # 'job_task_id',
            'status',
            'job_id',
            'secret_id',
            'collector_id',
            'provider',
            'service_account_id',
            'project_id',
            'domain_id',
            'created_at',
            'finished_at'
        ]
    }



import logging

from spaceone.core import config
from spaceone.core.service import *
from spaceone.inventory.error import *
from spaceone.inventory.model.cloud_service_query_set_model import CloudServiceQuerySet
from spaceone.inventory.manager.cloud_service_query_set_manager import CloudServiceQuerySetManager
from spaceone.inventory.manager.identity_manager import IdentityManager


_LOGGER = logging.getLogger(__name__)
_KEYWORD_FILTER = ['query_set_id', 'name']


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class CloudServiceQuerySetService(BaseService):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.cloud_svc_query_set_mgr: CloudServiceQuerySetManager = self.locator.get_manager('CloudServiceQuerySetManager')

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['name', 'query_options', 'query_options.fields', 'provider', 'cloud_service_group',
                     'cloud_service_type', 'domain_id'])
    def create(self, params):
        """ Create Cloud Service Query Set
        Args:
            params (dict): {
                    'name': 'str',
                    'query_options': 'dict',
                    'unit': 'dict',
                    'provider': 'str',
                    'cloud_service_group': 'str',
                    'cloud_service_type': 'str',
                    'tags': 'dict',
                    'domain_id': 'str'
                }

        Returns:
            cloud_service_query_set_vo (object)

        """

        params['query_type'] = 'CUSTOM'

        return self.cloud_svc_query_set_mgr.create_cloud_service_query_set(params)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['query_set_id', 'domain_id'])
    def update(self, params):
        """ Update Cloud Service Query Set
        Args:
            params (dict): {
                    'query_set_id': 'str',
                    'name': 'str',
                    'query_options': 'dict',
                    'unit': 'dict',
                    'tags': 'dict',
                    'domain_id': 'str'
                }

        Returns:
            cloud_service_query_set_vo (object)

        """

        cloud_svc_query_set_vo: CloudServiceQuerySet = \
            self.cloud_svc_query_set_mgr.get_cloud_service_query_set(params['query_set_id'], params['domain_id'])

        if cloud_svc_query_set_vo.query_type == 'MANAGED':
            raise ERROR_NOT_ALLOWED_QUERY_TYPE()

        return self.cloud_svc_query_set_mgr.update_cloud_service_query_set_by_vo(params, cloud_svc_query_set_vo)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['query_set_id', 'domain_id'])
    def delete(self, params):
        """ Delete Cloud Service Query Set
        Args:
            params (dict): {
                    'query_set_id': 'str',
                    'domain_id': 'str'
                }

        Returns:
            None

        """

        cloud_svc_query_set_vo: CloudServiceQuerySet = \
            self.cloud_svc_query_set_mgr.get_cloud_service_query_set(params['query_set_id'], params['domain_id'])

        if cloud_svc_query_set_vo.query_type == 'MANAGED':
            raise ERROR_NOT_ALLOWED_QUERY_TYPE()

        self.cloud_svc_query_set_mgr.delete_cloud_service_query_set_by_vo(cloud_svc_query_set_vo)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['query_set_id', 'domain_id'])
    def run(self, params):
        """ Run Query Set Manually and Save Results
        Args:
            params (dict): {
                    'query_set_id': 'str',
                    'domain_id': 'str'
                }

        Returns:
            None

        """

        cloud_svc_query_set_vo: CloudServiceQuerySet = \
            self.cloud_svc_query_set_mgr.get_cloud_service_query_set(params['query_set_id'], params['domain_id'])

        self.cloud_svc_query_set_mgr.run_cloud_service_query_set(cloud_svc_query_set_vo)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['query_set_id', 'domain_id'])
    def test(self, params):
        """ Test Query Set Manually
        Args:
            params (dict): {
                    'query_set_id': 'str',
                    'domain_id': 'str'
                }

        Returns:
            values (list) : 'list of analyze data'

        """

        cloud_svc_query_set_vo: CloudServiceQuerySet = \
            self.cloud_svc_query_set_mgr.get_cloud_service_query_set(params['query_set_id'], params['domain_id'])

        return self.cloud_svc_query_set_mgr.test_cloud_service_query_set(cloud_svc_query_set_vo)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['query_set_id', 'domain_id'])
    def enable(self, params):
        """ Enable Cloud Service Query Set
        Args:
            params (dict): {
                    'query_set_id': 'str',
                    'domain_id': 'str'
                }

        Returns:
            cloud_service_query_set_vo (object)

        """

        cloud_svc_query_set_vo: CloudServiceQuerySet = \
            self.cloud_svc_query_set_mgr.get_cloud_service_query_set(params['query_set_id'], params['domain_id'])

        if cloud_svc_query_set_vo.query_type == 'MANAGED':
            raise ERROR_NOT_ALLOWED_QUERY_TYPE()

        return self.cloud_svc_query_set_mgr.update_cloud_service_query_set_by_vo({'state': 'ENABLED'},
                                                                                 cloud_svc_query_set_vo)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['query_set_id', 'domain_id'])
    def disable(self, params):
        """ Disable Cloud Service Query Set
        Args:
            params (dict): {
                    'query_set_id': 'str',
                    'domain_id': 'str'
                }

        Returns:
            cloud_service_query_set_vo (object)

        """

        cloud_svc_query_set_vo: CloudServiceQuerySet = \
            self.cloud_svc_query_set_mgr.get_cloud_service_query_set(params['query_set_id'], params['domain_id'])

        if cloud_svc_query_set_vo.query_type == 'MANAGED':
            raise ERROR_NOT_ALLOWED_QUERY_TYPE()

        return self.cloud_svc_query_set_mgr.update_cloud_service_query_set_by_vo({'state': 'DISABLED'},
                                                                                 cloud_svc_query_set_vo)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['query_set_id', 'domain_id'])
    def get(self, params):
        """ Get Cloud Service Query Set
        Args:
            params (dict): {
                    'query_set_id': 'str',
                    'domain_id': 'str',
                    'only': 'list'
                }

        Returns:
            cloud_service_type_vo (object)

        """

        return self.cloud_svc_query_set_mgr.get_cloud_service_query_set(params['query_set_id'],
                                                                        params['domain_id'], params.get('only'))

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['domain_id'])
    @append_query_filter(['query_set_id', 'name', 'state', 'query_type', 'provider', 'cloud_service_group',
                          'cloud_service_type', 'domain_id'])
    @append_keyword_filter(_KEYWORD_FILTER)
    @set_query_page_limit(1000)
    def list(self, params):
        """ List Cloud Service Query Sets
        Args:
            params (dict): {
                    'query_set_id': 'str',
                    'name': 'str',
                    'state': 'str',
                    'query_type': 'str',
                    'provider': 'str',
                    'cloud_service_group': 'str',
                    'cloud_service_type': 'str',
                    'domain_id': 'str',
                    'query': 'dict (spaceone.api.core.v1.Query)'
                }

        Returns:
            results (list)
            total_count (int)

        """
        query = params.get('query', {})
        return self.cloud_svc_query_set_mgr.list_cloud_service_query_sets(query)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['query', 'domain_id'])
    @append_query_filter(['domain_id'])
    @append_keyword_filter(_KEYWORD_FILTER)
    def stat(self, params):
        """ Get Cloud Service Query Set Statistics
        Args:
            params (dict): {
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)'
            }

        Returns:
            values (list) : 'list of statistics data'

        """

        query = params.get('query', {})
        return self.cloud_svc_query_set_mgr.stat_cloud_service_query_sets(query)

    @transaction(append_meta={'authorization.scope': 'SYSTEM'})
    @check_required(['domain_id'])
    def run_query_sets_by_domain(self, params):
        """ Run cloud service query sets by domain_id

        Args:
            params (dict): {
                'domain_id': 'str'
            }

        Returns:
            None
        """

        domain_id = params['domain_id']
        query_set_vos = self.cloud_svc_query_set_mgr.filter_cloud_service_query_sets(domain_id=domain_id)
        for query_set_vo in query_set_vos:
            self.cloud_svc_query_set_mgr.run_cloud_service_query_set(query_set_vo)

    @transaction(append_meta={'authorization.scope': 'SYSTEM'})
    def run_all_query_sets(self, params):
        """ Run all cloud service query sets

        Args:
            params (dict): {}

        Returns:
            None
        """

        system_token = config.get_global('TOKEN')

        for domain_info in self._get_all_domains_info(system_token):
            domain_id = domain_info['domain_id']
            try:
                self.cloud_svc_query_set_mgr.push_task(domain_id, system_token)
            except Exception as e:
                _LOGGER.error(f'[run_query_sets_by_domain] query error({domain_id}): {e}', exc_info=True)

    def _get_all_domains_info(self, system_token):
        identity_mgr: IdentityManager = self.locator.get_manager('IdentityManager', token=system_token)
        response = identity_mgr.list_domains({
            'only': ['domain_id'],
            'filter': [{'k': 'state', 'v': 'ENABLED', 'o': 'eq'}]
        })

        return response.get('results', [])

    def _list_query_sets_by_domain(self, domain_id):
        return self.cloud_svc_query_set_mgr.filter_cloud_service_query_sets(domain_id=domain_id)



from spaceone.core.error import *


class ERROR_NOT_ALLOWED_QUERY_TYPE(ERROR_INVALID_ARGUMENT):
    _message = 'Managed query set does not allow to update or delete.'


class ERROR_CLOUD_SERVICE_QUERY_SET_STATE(ERROR_INVALID_ARGUMENT):
    _message = 'Query set is not available. (state = {state})'


class ERROR_CLOUD_SERVICE_QUERY_SET_RUN_FAILED(ERROR_UNKNOWN):
    _message = 'Query set run failed. (query_set_id = {query_set_id})'



"""
Deprecated:
  Not used. Integrated with SpaceConnector.
"""

import logging

from spaceone.core.connector import BaseConnector
from spaceone.core import pygrpc
from spaceone.core.utils import parse_grpc_endpoint
from spaceone.core.error import *

__all__ = ['SecretConnector']
_LOGGER = logging.getLogger(__name__)


class SecretConnector(BaseConnector):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        if 'endpoint' not in self.config:
            raise ERROR_WRONG_CONFIGURATION(key='endpoint')

        if len(self.config['endpoint']) > 1:
            raise ERROR_WRONG_CONFIGURATION(key='too many endpoint')

        for version, uri in self.config['endpoint'].items():
            e = parse_grpc_endpoint(uri)
            self.client = pygrpc.client(endpoint=e['endpoint'], ssl_enabled=e['ssl_enabled'])

    def get_secret(self, secret_id, domain_id):
        return self.client.Secret.get({'secret_id': secret_id, 'domain_id': domain_id},
                                      metadata=self.transaction.get_connection_meta())

    def list_secrets_by_secret_group_id(self, secret_group_id, domain_id):
        return self.client.Secret.list({'secret_group_id': secret_group_id, 'domain_id': domain_id},
                                       metadata=self.transaction.get_connection_meta())

    def list_secrets_by_provider(self, provider, domain_id):
        return self.client.Secret.list({'provider': provider, 'domain_id': domain_id},
                                       metadata=self.transaction.get_connection_meta())

    def get_secret_data(self, secret_id, domain_id):
        return self.client.Secret.get_data({'secret_id': secret_id, 'domain_id': domain_id},
                                           metadata=self.transaction.get_connection_meta())



import logging
import copy
from datetime import datetime

from spaceone.core.service import *
from spaceone.core import utils
from spaceone.inventory.model.cloud_service_model import CloudService, CollectionInfo
from spaceone.inventory.manager.cloud_service_manager import CloudServiceManager
from spaceone.inventory.manager.region_manager import RegionManager
from spaceone.inventory.manager.identity_manager import IdentityManager
from spaceone.inventory.manager.resource_group_manager import ResourceGroupManager
from spaceone.inventory.manager.change_history_manager import ChangeHistoryManager
from spaceone.inventory.manager.collection_state_manager import CollectionStateManager
from spaceone.inventory.manager.note_manager import NoteManager
from spaceone.inventory.manager.collector_rule_manager import CollectorRuleManager
from spaceone.inventory.error import *

_KEYWORD_FILTER = ['cloud_service_id', 'name', 'ip_addresses', 'cloud_service_group', 'cloud_service_type',
                   'reference.resource_id']

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class CloudServiceService(BaseService):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.cloud_svc_mgr: CloudServiceManager = self.locator.get_manager('CloudServiceManager')
        self.region_mgr: RegionManager = self.locator.get_manager('RegionManager')
        self.identity_mgr: IdentityManager = self.locator.get_manager('IdentityManager')
        self.collector_rule_mgr: CollectorRuleManager = self.locator.get_manager('CollectorRuleManager')
        self.collector_id = self.transaction.get_meta('collector_id')
        self.job_id = self.transaction.get_meta('job_id')
        self.plugin_id = self.transaction.get_meta('plugin_id')
        self.service_account_id = self.transaction.get_meta('secret.service_account_id')

    @transaction(append_meta={
        'authorization.scope': 'PROJECT',
        'authorization.require_project_id': True
    })
    def create(self, params):
        """
        Args:
            params (dict): {
                    'cloud_service_type': 'str',
                    'cloud_service_group': 'str',
                    'provider': 'str',
                    'name': 'str',
                    'account': 'str',
                    'instance_type': 'str',
                    'instance_size': 'float',
                    'ip_addresses': 'list',
                    'data': 'dict',
                    'metadata': 'dict',
                    'reference': 'dict',
                    'tags': 'list or dict',
                    'region_code': 'str',
                    'project_id': 'str',
                    'domain_id': 'str'
                }

        Returns:
            cloud_service_vo (object)

        """
        return self.create_resource(params)

    @check_required(['cloud_service_type', 'cloud_service_group', 'provider', 'data', 'domain_id'])
    def create_resource(self, params):
        ch_mgr: ChangeHistoryManager = self.locator.get_manager('ChangeHistoryManager')

        domain_id = params['domain_id']
        secret_project_id = self.transaction.get_meta('secret.project_id')
        provider = params['provider']

        if 'instance_size' in params:
            if not isinstance(params['instance_size'], float):
                raise ERROR_INVALID_PARAMETER_TYPE(key='instance_size', type='float')

        if 'tags' in params:
            params['tags'] = self._convert_tags_to_dict(params['tags'])

        # Change data through Collector Rule
        if self._is_created_by_collector():
            params = self.collector_rule_mgr.change_cloud_service_data(self.collector_id, domain_id, params)

        if 'tags' in params:
            params['tags'], params['tag_keys'] = self._convert_tags_to_hash(params['tags'], provider)

        if 'project_id' in params:
            self.identity_mgr.get_project(params['project_id'], domain_id)
        elif secret_project_id:
            params['project_id'] = secret_project_id

        params['ref_cloud_service_type'] = self._make_cloud_service_type_key(params)

        if 'region_code' in params:
            params['ref_region'] = self._make_region_key(params, params['provider'])

        if 'metadata' in params:
            params['metadata'] = self._convert_metadata(params['metadata'], provider)

        params['collection_info'] = self._get_collection_info(provider)

        cloud_svc_vo = self.cloud_svc_mgr.create_cloud_service(params)

        # Create New History
        ch_mgr.add_new_history(cloud_svc_vo, params)

        # Create Collection State
        state_mgr: CollectionStateManager = self.locator.get_manager('CollectionStateManager')
        state_mgr.create_collection_state(cloud_svc_vo.cloud_service_id, 'inventory.CloudService', domain_id)

        return cloud_svc_vo

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    def update(self, params):
        """
        Args:
            params (dict): {
                    'cloud_service_id': 'str',
                    'name': 'str',
                    'account': 'str',
                    'instance_type': 'str',
                    'instance_size': 'float',
                    'ip_addresses': 'list',
                    'data': 'dict',
                    'metadata': 'dict',
                    'reference': 'dict',
                    'tags': 'list or dict',
                    'region_code': 'str',
                    'project_id': 'str',
                    'domain_id': 'str',
                    'release_project': 'bool',
                    'release_region': 'bool'
                }

        Returns:
            cloud_service_vo (object)
        """
        return self.update_resource(params)

    @check_required(['cloud_service_id', 'domain_id'])
    def update_resource(self, params):
        ch_mgr: ChangeHistoryManager = self.locator.get_manager('ChangeHistoryManager')

        secret_project_id = self.transaction.get_meta('secret.project_id')

        cloud_service_id = params['cloud_service_id']
        domain_id = params['domain_id']
        release_region = params.get('release_region', False)
        release_project = params.get('release_project', False)
        provider = self._get_provider_from_meta()

        if 'ip_addresses' in params and params['ip_addresses'] is None:
            del params['ip_addresses']

        if 'instance_size' in params:
            if not isinstance(params['instance_size'], float):
                raise ERROR_INVALID_PARAMETER_TYPE(key='instance_size', type='float')

        if 'tags' in params:
            params['tags'] = self._convert_tags_to_dict(params['tags'])

        # Change data through Collector Rule
        if self._is_created_by_collector():
            params = self.collector_rule_mgr.change_cloud_service_data(self.collector_id, domain_id, params)

        cloud_svc_vo: CloudService = self.cloud_svc_mgr.get_cloud_service(cloud_service_id, domain_id)

        if release_project:
            params['project_id'] = None
        elif 'project_id' in params:
            self.identity_mgr.get_project(params['project_id'], domain_id)
        elif secret_project_id and secret_project_id != cloud_svc_vo.project_id:
            params['project_id'] = secret_project_id

        if release_region:
            params.update({
                'region_code': None,
                'ref_region': None
            })
        elif 'region_code' in params:
            params['ref_region'] = self._make_region_key(params, cloud_svc_vo.provider)

        old_cloud_svc_data = dict(cloud_svc_vo.to_dict())

        if 'tags' in params:
            old_tags = old_cloud_svc_data.get('tags', {})
            old_tag_keys = old_cloud_svc_data.get('tag_keys', {})
            new_tags, new_tag_keys = self._convert_tags_to_hash(params['tags'], provider)

            if self._is_different_data(new_tags, old_tags, provider):
                old_tags.update(new_tags)
                old_tag_keys.update(new_tag_keys)
                params['tags'] = old_tags
                params['tag_keys'] = old_tag_keys
            else:
                del params['tags']

        if 'metadata' in params:
            old_metadata = old_cloud_svc_data.get('metadata', {})
            new_metadata = self._convert_metadata(params['metadata'], provider)

            if self._is_different_data(new_metadata, old_metadata, provider):
                old_metadata.update(new_metadata)
                params['metadata'] = old_metadata
            else:
                del params['metadata']

        old_collection_info = old_cloud_svc_data.get('collection_info', [])
        params['collection_info'] = self._get_collection_info(provider, old_collection_info)

        params = self.cloud_svc_mgr.merge_data(params, old_cloud_svc_data)

        cloud_svc_vo = self.cloud_svc_mgr.update_cloud_service_by_vo(params, cloud_svc_vo)

        # Create Update History
        ch_mgr.add_update_history(cloud_svc_vo, params, old_cloud_svc_data)

        # Update Collection History
        state_mgr: CollectionStateManager = self.locator.get_manager('CollectionStateManager')
        state_vo = state_mgr.get_collection_state(cloud_service_id, domain_id)
        if state_vo:
            state_mgr.reset_collection_state(state_vo)
        else:
            state_mgr.create_collection_state(cloud_service_id, 'inventory.CloudService', domain_id)

        if 'project_id' in params:
            note_mgr: NoteManager = self.locator.get_manager('NoteManager')

            # Update Project ID from Notes
            note_vos = note_mgr.filter_notes(cloud_service_id=cloud_service_id, domain_id=domain_id)
            note_vos.update({'project_id': params['project_id']})

        return cloud_svc_vo

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    def delete(self, params):
        self.delete_resource(params)

    @check_required(['cloud_service_id', 'domain_id'])
    def delete_resource(self, params):
        """
        Args:
        params (dict): {
            'cloud_service_id': 'str',
            'domain_id': 'str'
        }
        Returns:
            None
        """
        ch_mgr: ChangeHistoryManager = self.locator.get_manager('ChangeHistoryManager')

        cloud_service_id = params['cloud_service_id']
        domain_id = params['domain_id']

        cloud_svc_vo: CloudService = self.cloud_svc_mgr.get_cloud_service(cloud_service_id, domain_id)

        self.cloud_svc_mgr.delete_cloud_service_by_vo(cloud_svc_vo)

        # Create Update History
        ch_mgr.add_delete_history(cloud_svc_vo)

        # Cascade Delete Collection State
        state_mgr: CollectionStateManager = self.locator.get_manager('CollectionStateManager')
        state_mgr.delete_collection_state_by_resource_id(cloud_service_id, domain_id)

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['cloud_service_id', 'domain_id'])
    def get(self, params):
        """
        Args:
            params (dict): {
                    'cloud_service_id': 'str',
                    'domain_id': 'str',
                    'only': 'list'
                }

        Returns:
            cloud_service_vo (object)

        """

        return self.cloud_svc_mgr.get_cloud_service(params['cloud_service_id'], params['domain_id'],
                                                    only=params.get('only'))

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['domain_id'])
    @append_query_filter(['cloud_service_id', 'name', 'state', 'account', 'instance_type', 'cloud_service_type',
                          'cloud_service_group', 'provider', 'region_code', 'project_id', 'project_group_id',
                          'domain_id', 'user_projects', 'ip_address'])
    @append_keyword_filter(_KEYWORD_FILTER)
    @set_query_page_limit(1000)
    def list(self, params):
        """
        Args:
            params (dict): {
                    'cloud_service_id': 'str',
                    'name': 'str',
                    'state': 'str',
                    'account': 'str',
                    'instance_type': 'str',
                    'cloud_service_type': 'str',
                    'cloud_service_group': 'str',
                    'provider': 'str',
                    'region_code': 'str',
                    'project_id': 'str',
                    'project_group_id': 'str',
                    'domain_id': 'str',
                    'query': 'dict (spaceone.api.core.v1.Query)',
                    'user_projects': 'list', // from meta
                    'ip_address': 'str'
                }

        Returns:
            results (list)
            total_count (int)

        """
        query = params.get('query', {})
        # query = self._append_resource_group_filter(query, params['domain_id'])
        query = self._change_project_group_filter(query, params['domain_id'])
        query = self._change_filter_tags(query)
        query = self._change_only_tags(query)
        query = self._change_sort_tags(query)

        return self.cloud_svc_mgr.list_cloud_services(query)

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['query', 'query.fields', 'domain_id'])
    @append_query_filter(['domain_id', 'user_projects'])
    @append_keyword_filter(_KEYWORD_FILTER)
    @set_query_page_limit(1000)
    def analyze(self, params):
        """
        Args:
            params (dict): {
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.AnalyzeQuery)',
                'user_projects': 'list', // from meta
            }

        Returns:
            results (list) : 'list of analyze data'

        """

        query = params.get('query', {})
        # query = self._append_resource_group_filter(query, params['domain_id'])
        query = self._change_project_group_filter(query, params['domain_id'])
        query = self._change_filter_tags(query)

        return self.cloud_svc_mgr.analyze_cloud_services(query)

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['query', 'domain_id'])
    @append_query_filter(['domain_id', 'user_projects'])
    @append_keyword_filter(_KEYWORD_FILTER)
    def stat(self, params):
        """
        Args:
            params (dict): {
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)',
                'user_projects': 'list', // from meta
            }

        Returns:
            results (list) : 'list of statistics data'

        """

        query = params.get('query', {})
        # query = self._append_resource_group_filter(query, params['domain_id'])
        query = self._change_project_group_filter(query, params['domain_id'])
        query = self._change_filter_tags(query)
        query = self._change_distinct_tags(query)

        return self.cloud_svc_mgr.stat_cloud_services(query)

    def _append_resource_group_filter(self, query, domain_id):
        change_filter = []

        for condition in query.get('filter', []):
            key = condition.get('k', condition.get('key'))
            value = condition.get('v', condition.get('value'))
            operator = condition.get('o', condition.get('operator'))

            if key == 'resource_group_id':
                cloud_service_ids = None

                if operator in ['not', 'not_contain', 'not_in', 'not_contain_in']:
                    resource_group_operator = 'not_in'
                else:
                    resource_group_operator = 'in'

                if operator in ['eq', 'not', 'contain', 'not_contain']:
                    cloud_service_ids = self._get_cloud_service_ids_from_resource_group_id(value, domain_id)
                elif operator in ['in', 'not_in', 'contain_in', 'not_contain_in'] and isinstance(value, list):
                    cloud_service_ids = []
                    for v in value:
                        cloud_service_ids += self._get_cloud_service_ids_from_resource_group_id(v, domain_id)

                if cloud_service_ids is not None:
                    change_filter.append({
                        'k': 'cloud_service_id',
                        'v': list(set(cloud_service_ids)),
                        'o': resource_group_operator
                    })

            else:
                change_filter.append(condition)

        query['filter'] = change_filter
        return query

    def _get_cloud_service_ids_from_resource_group_id(self, resource_group_id, domain_id):
        resource_type = 'inventory.CloudService'
        rg_mgr: ResourceGroupManager = self.locator.get_manager('ResourceGroupManager')

        resource_group_filters = rg_mgr.get_resource_group_filter(resource_group_id, resource_type, domain_id,
                                                                  _KEYWORD_FILTER)
        cloud_service_ids = []
        for resource_group_query in resource_group_filters:
            resource_group_query['distinct'] = 'cloud_service_id'
            result = self.cloud_svc_mgr.stat_cloud_services(resource_group_query)
            cloud_service_ids += result.get('results', [])
        return cloud_service_ids

    def _change_project_group_filter(self, query, domain_id):
        change_filter = []

        project_group_query = {
            'filter': [],
            'only': ['project_group_id']
        }

        for condition in query.get('filter', []):
            key = condition.get('key', condition.get('k'))
            value = condition.get('value', condition.get('v'))
            operator = condition.get('operator', condition.get('o'))

            if not all([key, operator]):
                raise ERROR_DB_QUERY(reason='filter condition should have key, value and operator.')

            if key == 'project_group_id':
                project_group_query['filter'].append(condition)
            else:
                change_filter.append(condition)

        if len(project_group_query['filter']) > 0:
            identity_mgr: IdentityManager = self.locator.get_manager('IdentityManager')
            response = identity_mgr.list_project_groups(project_group_query, domain_id)
            project_group_ids = []
            project_ids = []
            for project_group_info in response.get('results', []):
                project_group_ids.append(project_group_info['project_group_id'])

            for project_group_id in project_group_ids:
                response = identity_mgr.list_projects_in_project_group(project_group_id, domain_id, True,
                                                                       {'only': ['project_id']})
                for project_info in response.get('results', []):
                    if project_info['project_id'] not in project_ids:
                        project_ids.append(project_info['project_id'])

            change_filter.append({'k': 'project_id', 'v': project_ids, 'o': 'in'})

        query['filter'] = change_filter
        return query

    @staticmethod
    def _make_cloud_service_type_key(resource_data):
        return f'{resource_data["domain_id"]}.{resource_data["provider"]}.' \
               f'{resource_data["cloud_service_group"]}.{resource_data["cloud_service_type"]}'

    @staticmethod
    def _make_region_key(resource_data, provider):
        return f'{resource_data["domain_id"]}.{provider}.{resource_data["region_code"]}'

    @staticmethod
    def _convert_metadata(metadata, provider):
        return {provider: copy.deepcopy(metadata)}

    def _get_collection_info(self, provider, collections: CollectionInfo = None):
        collections = collections or []

        collector_id = self.transaction.get_meta('collector_id')
        secret_id = self.transaction.get_meta('secret.secret_id')
        service_account_id = self.transaction.get_meta('secret.service_account_id')

        new_collection = {
            'provider': provider,
            'collector_id': collector_id,
            'secret_id': secret_id,
            'service_account_id': service_account_id,
            'last_collected_at': datetime.utcnow()
        }

        merged_collections = []
        if collections:
            for collection in collections:
                if collection['provider'] == provider:
                    merged_collections.append(new_collection)
                else:
                    merged_collections.append(collection)
        else:
            merged_collections.append(new_collection)

        return merged_collections

    @staticmethod
    def _convert_tags_to_dict(tags):
        if isinstance(tags, list):
            dot_tags = utils.tags_to_dict(tags)
        elif isinstance(tags, dict):
            dot_tags = copy.deepcopy(tags)
        else:
            dot_tags = {}
        return dot_tags

    @staticmethod
    def _convert_tags_to_hash(dot_tags, provider):
        tag_keys = {provider: list(dot_tags.keys())}

        tags = {provider: {}}
        for key, value in dot_tags.items():
            hashed_key = utils.string_to_hash(key)
            tags[provider][hashed_key] = {'key': key, 'value': value}

        return tags, tag_keys

    @staticmethod
    def _is_different_data(new_data, old_data, provider):
        if new_data[provider] != old_data.get(provider):
            return True
        else:
            return False

    def _get_provider_from_meta(self):
        if self._is_created_by_collector():
            return self.transaction.get_meta('secret.provider')
        else:
            return 'custom'

    def _is_created_by_collector(self):
        return self.collector_id and self.job_id and self.service_account_id and self.plugin_id

    def _change_filter_tags(self, query):
        change_filter = []

        for condition in query.get('filter', []):
            key = condition.get('k', condition.get('key'))
            value = condition.get('v', condition.get('value'))
            operator = condition.get('o', condition.get('operator'))

            if key.startswith('tags.'):
                hashed_key = self._get_hashed_key(key)

                change_filter.append({
                    'key': hashed_key,
                    'value': value,
                    'operator': operator
                })

            else:
                change_filter.append(condition)
        query['filter'] = change_filter
        return query

    def _change_only_tags(self, query):
        change_only_tags = []
        if 'only' in query:
            for key in query.get('only', []):
                if key.startswith('tags.'):
                    hashed_key = self._get_hashed_key(key, only=True)
                    change_only_tags.append(hashed_key)
                else:
                    change_only_tags.append(key)
            query['only'] = change_only_tags

        return query

    def _change_distinct_tags(self, query):
        if 'distinct' in query:
            distinct_key = query['distinct']
            if distinct_key.startswith('tags.'):
                hashed_key = self._get_hashed_key(distinct_key)
                query['distinct'] = hashed_key

        return query

    def _change_sort_tags(self, query):
        if 'sort' in query:
            if 'keys' in query['sort']:
                change_filter = []
                sort_keys = query['sort'].get('keys', [])
                for condition in sort_keys:
                    sort_key = condition.get('key', '')
                    desc = condition.get('desc', False)

                    if sort_key.startswith('tags.'):
                        hashed_key = self._get_hashed_key(sort_key)

                        change_filter.append({
                            'key': hashed_key,
                            'desc': desc
                        })

                    else:
                        change_filter.append({
                            'key': sort_key,
                            'desc': desc
                        })
                query['sort']['keys'] = change_filter

            elif 'key' in query['sort']:
                change_filter = {}
                sort_key = query['sort']['key']
                desc = query['sort'].get('desc', False)

                if sort_key.startswith('tags.'):
                    hashed_key = self._get_hashed_key(sort_key)

                    change_filter.update({
                        'key': hashed_key,
                        'desc': desc
                    })

                else:
                    change_filter.update({
                        'key': sort_key,
                        'desc': desc
                    })
                query['sort'] = change_filter
        return query

    @staticmethod
    def _get_hashed_key(key, only=False):
        if key.count('.') < 2:
            return key

        prefix, provider, key = key.split('.', 2)
        hash_key = utils.string_to_hash(key)
        if only:
            return f'{prefix}.{provider}.{hash_key}'
        else:
            return f'{prefix}.{provider}.{hash_key}.value'



from spaceone.core.service import *
from spaceone.inventory.model.job_model import Job
from spaceone.inventory.manager.job_manager import JobManager


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class JobService(BaseService):

    def __init__(self, metadata):
        super().__init__(metadata)
        self.job_mgr: JobManager = self.locator.get_manager('JobManager')

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['job_id', 'domain_id'])
    def delete(self, params):
        """
        Args:
            params (dict): {
                'job_id': 'str',
                'domain_id': 'str'
            }

        Returns:
            None
        """
        job_id = params['job_id']
        domain_id = params['domain_id']

        job_vo: Job = self.job_mgr.get_job(job_id, domain_id)
        self.job_mgr.delete_job_by_vo(job_vo)

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['job_id', 'domain_id'])
    def get(self, params):
        """
        Args:
            params (dict): {
                'job_id': 'str',
                'domain_id': 'str',
                'only': 'list'
            }

        Returns:
            job_vo (object)
        """

        return self.job_mgr.get_job(params['job_id'], params['domain_id'], params.get('only'))

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['domain_id'])
    @change_only_key({'collector_info': 'collector'}, key_path='query.only')
    @append_query_filter(['job_id', 'status', 'collector_id', 'secret_id', 'domain_id', 'user_projects'])
    @append_keyword_filter(['job_id'])
    def list(self, params):
        """
        Args:
            params (dict): {
                'job_id': 'str',
                'status': 'str',
                'collector_id': 'dict',
                'project_id': 'str',
                'secret_id': 'str',
                'domain_id  ': 'str',
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)',
                'user_projects': 'list', // from meta
            }

        Returns:
            results (list)
            total_count (int)
        """
        query = params.get('query', {})

        # Temporary code for DB migration
        if 'only' in query:
            query['only'] += ['collector_id']

        return self.job_mgr.list_jobs(query)

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['query', 'query.fields', 'domain_id'])
    @append_query_filter(['domain_id', 'user_projects'])
    @append_keyword_filter(['job_id'])
    @set_query_page_limit(1000)
    def analyze(self, params):
        """
        Args:
            params (dict): {
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.AnalyzeQuery)',
                'user_projects': 'list', // from meta
            }

        Returns:
            values (list) : 'list of statistics data'
        """

        query = params.get('query', {})

        return self.job_mgr.analyze_jobs(query)

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['query', 'domain_id'])
    @append_query_filter(['domain_id', 'user_projects'])
    @append_keyword_filter(['job_id'])
    def stat(self, params):
        """
        Args:
            params (dict): {
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)',
                'user_projects': 'list', // from meta
            }

        Returns:
            values (list) : 'list of statistics data'
        """

        query = params.get('query', {})
        return self.job_mgr.stat_jobs(query)



import logging

from spaceone.core.manager import BaseManager
from spaceone.inventory.lib.resource_manager import ResourceManager
from spaceone.inventory.model.region_model import Region

_LOGGER = logging.getLogger(__name__)


class RegionManager(BaseManager, ResourceManager):

    resource_keys = ['region_id']
    query_method = 'list_regions'

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.region_model: Region = self.locator.get_model('Region')

    def create_region(self, params):
        def _rollback(region_vo):
            _LOGGER.info(f'[ROLLBACK] Delete region : {region_vo.name} ({region_vo.region_id})')
            region_vo.delete()

        region_vo: Region = self.region_model.create(params)
        self.transaction.add_rollback(_rollback, region_vo)

        return region_vo

    def update_region(self, params):
        return self.update_region_by_vo(params, self.get_region(params['region_id'], params['domain_id']))

    def update_region_by_vo(self, params, region_vo):
        def _rollback(old_data):
            _LOGGER.info(f'[ROLLBACK] Revert Data : {old_data["name"]} ({old_data["region_id"]})')
            region_vo.update(old_data)

        self.transaction.add_rollback(_rollback, region_vo.to_dict())
        return region_vo.update(params)

    def delete_region(self, region_id, domain_id):
        self.delete_region_by_vo(self.get_region(region_id, domain_id))

    def get_region(self, region_id, domain_id, only=None):
        return self.region_model.get(region_id=region_id, domain_id=domain_id, only=only)

    def get_region_from_code(self, region_code, region_type, domain_id, only=None):
        return self.region_model.get(region_code=region_code, region_type=region_type, domain_id=domain_id, only=only)

    def list_regions(self, query):
        return self.region_model.query(**query)

    def stat_regions(self, query):
        return self.region_model.stat(**query)

    @staticmethod
    def delete_region_by_vo(region_vo):
        region_vo.delete()



from mongoengine import *

from spaceone.core.model.mongo_model import MongoModel


class Note(MongoModel):
    note_id = StringField(max_length=40, generate_id='note', unique=True)
    note = StringField()
    record_id = StringField(max_length=40)
    cloud_service_id = StringField(max_length=40)
    created_by = StringField(max_length=40)
    project_id = StringField(max_length=255, default=None, null=True)
    domain_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)

    meta = {
        'updatable_fields': [
            'note'
        ],
        'minimal_fields': [
            'note_id',
            'note',
            'record_id',
            'cloud_service_id',
            'created_by',
            'project_id'
        ],
        'change_query_keys': {
            'user_projects': 'project_id'
        },
        'ordering': [
            '-created_at'
        ],
        'indexes': [
            'record_id',
            'cloud_service_id',
            'created_by',
            'project_id',
            'domain_id'
        ]
    }



import logging

from spaceone.core.manager import BaseManager
from spaceone.core.connector.space_connector import SpaceConnector
from spaceone.inventory.error import *
from spaceone.inventory.connector.repository_connector import RepositoryConnector

_LOGGER = logging.getLogger(__name__)


class RepositoryManager(BaseManager):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.repo_connector: SpaceConnector = self.locator.get_connector('SpaceConnector', service='repository')

    def get_plugin(self, plugin_id, domain_id):
        return self.repo_connector.dispatch('Plugin.get', {'plugin_id': plugin_id, 'domain_id': domain_id})

    def check_plugin_version(self, plugin_id, version, domain_id):
        response = self.repo_connector.dispatch('Plugin.get_versions', {'plugin_id': plugin_id, 'domain_id': domain_id})

        if version not in response.get('results', []):
            raise ERROR_INVALID_PLUGIN_VERSION(plugin_id=plugin_id, version=version)

    def list_schemas(self, query, domain_id):
        return self.repo_connector.dispatch('Schema.list', {'query': query, 'domain_id': domain_id})



"""
Deprecated
"""
import logging

from spaceone.core.manager import BaseManager

_LOGGER = logging.getLogger(__name__)

"""
Schedule
"""
class ScheduleManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.schedule_model = self.locator.get_model('Schedule')

    def create_schedule(self, params):
        def _rollback(schedule_vo):
            _LOGGER.info(f'[ROLLBACK] Delete schedule : {schedule_vo.name} ({schedule_vo.schedule_id})')
            schedule_vo.delete()

        schedule_vo = self.schedule_model.create(params)
        self.transaction.add_rollback(_rollback, schedule_vo)
        return schedule_vo

    def delete_schedule(self, schedule_id, domain_id):
        schedule_vo = self.schedule_model.get(schedule_id=schedule_id, domain_id=domain_id)
        schedule_vo.delete()

    def get_schedule(self, schedule_id, domain_id):
        schedule_vo = self.schedule_model.get(schedule_id=schedule_id, domain_id=domain_id)
        return schedule_vo

    def list_schedules(self, query):
        return self.schedule_model.query(**query)

    def delete_by_collector_id(self, collector_id, domain_id):
        """ Delete all schedules related with collector
        """
        query = {
            'filter': [
                {'k': 'collector_id', 'v': collector_id, 'o': 'eq'},
                {'k': 'domain_id', 'v': domain_id, 'o': 'eq'}
            ]
        }
        schedule_vos, total_count = self.list_schedules(query)
        _LOGGER.debug(f'[delete_by_collector_id] found: {total_count}')
        for schedule_vo in schedule_vos:
            _LOGGER.debug(f'[delete_by_collector_id] delete schedule: {schedule_vo.schedule_id}')
            self.delete_schedule(schedule_vo.schedule_id, domain_id)
